#!/usr/bin/perl

#
# this script sanitizes "ipsec look" output more thoroughly than 
# ipsec-look-sanitize. The former is suitable for static conns,
# but dynamic ones come with random ESP SPIs, IVs, and presents the
# data in seemingly random order.
#

$inlook=0;

while(<>) {
  # look for "ipsec look" first.

  if(!$inlook && / ipsec look/) {
    $inlook=1;

    # also reset tunnel list.
    @tunnels=();
    @esps=();
    %spigrp=undef;

    # eat the date that is on the next line.
    print;

    $_=<>;
    # east Thu Nov  7 22:22:34 GMT 2002
    s/(.*) ... ... .. ..:..:.. GMT ..../\1 NOW/;

    $ineroute=1;
    $intncfg=0;
    $inspigrp=0;
    $inroute=0;
    print;
    next;
    
  } elsif($!inlook) {
    print;
    next;
  } elsif(/^(\S*)\:\~\#/) {
    $inlook=0;
    print;
    next;
  }
  
  ## okay we are in the look itself.

  if($ineroute) {
    # okay, in the eroute display, we have to sanitize any ESPs that we see.
    # 192.1.2.23/32      -> 192.0.1.3/32       => tun0x1002@192.1.2.45 esp0x515a1ad5@192.1.2.45  (8)
    # 11111111111111111111111111111111111111111111111112222 333333333344444455555555 666666666677777
    #
    # regexp fills $1-$7 as above
    # 
    if(m,^([\d.]+/\d+\s+\-\>\s+[\d.]+/\d+\s+\=\> tun0x)(\d+)@([\d.]+)( esp0x)([0-9a-f]{8})@([\d.]+)(.*)$,) {
      # okay, rebuild $_ with sanitized versions, and record the order of the
      # tunnels by dest IP $3.
      
      push(@tunnels, $2);
      push(@esps,    $3);
      print $1."IPIP@".$3.$4."KLIPSPIK@".$6.$7;
      next;
    } elsif(/^ipsec.*->eth.* mtu=.*(.*)->.*/) {
      $ineroute=0;
      $intncfg=1;
      print;
      next;
    } else {
      print;
      next;
    }
  }

  if($intncfg) {
    if(/^ipsec.*->eth.* mtu=.*(.*)->.*/) {
      print;
      next;
    }
    
    $intncfg=0;
    $inspigrp=1;
    # fall through
  }

  if($inspigrp) {
    # suck up all of the spigrp entries, and sanitize them and emit them in
    # the order that we recorded above, and order in, then out.
    # esp0x515a1ad5@192.1.2.45 ESP_3DES_HMAC_MD5: dir=out src=192.1.2.23 iv_bits=64bits iv=0x64c818022e5c1fa9 ooowin=64 seq=8 alen=128 aklen=128 eklen=192 life(c,s,h)=bytes(1088,0,0)addtime(26,0,0)usetime(26,0,0)packets(8,0,0) idle=13 refcount=4 ref=15
    # 1111122222222 3333333333 444-end.
    #
    # note that other sanitizing is applied to $4.
    if(m,^(esp0x)([0-9a-f]{1,8})@([\d.]+)( ESP_.*),) {
      
      $rest = $4;
      $esp1   = $1;
      $spinum = $2;
      $spiip  = $3;

      $rest =~ s/iv=0x[0-9a-f]{16}/iv=0xIVISFORRANDOM000/;
      $rest =~ s/life\(c,s,h\)=bytes\(.*\) //g;
      $rest =~ s/idle=.* //g;
      $rest =~ s/refcount=.* //g;
      $rest =~ s/ref=.* //g;
      $rest =~ s/addtime\(.*,.*,.*\)//;

      $espline=$esp1."KLIPSPIK@".$spiip.$rest;

      # if dir=in, then add the IP to the list.
      if($rest =~ m,dir=in,) {
	push(@tunnels, $spiip);
      }

      $espgrp{$spiip}=$espline;
    }
    # tun0x1002@192.1.2.45 IPIP: dir=out src=192.1.2.23 life(c,s,h)=bytes(832,0,0)addtime(26,0,0)usetime(26,0,0)packets(8,0,0) idle=13 refcount=4 ref=16
    # 111112222 3333333333 44444444444444444444444444

    elsif(m,^(tun0x)([0-9a-f]{1,8})@([\d.]+)( IPIP:.*),) {
      $rest = $4;
      $tun1 = $1;
      $spinum = $2;
      $spiip  = $3;

      $rest =~ s/iv=0x[0-9a-f]{16}/iv=0xIVISFORRANDOM000/;
      $rest =~ s/life\(c,s,h\)=bytes\(.*\) //g;
      $rest =~ s/idle=.* //g;
      $rest =~ s/refcount=.* //g;
      $rest =~ s/ref=.* //g;
      $rest =~ s/addtime\(.*,.*,.*\)//;

      $tunline=$tun1."TUN#@".$spiip.$rest;
      
      $tungrp{$spiip}=$tunline;
    } elsif(/^Destination/) {
      $inspigrp=0;
      $inroute=1;

      # dump the esp/spi table.

      foreach $ip (@tunnels) {
	
      
    
  
  
  
